#include <allegro.h>

#include "defines.h"

#include "enemy.h"
#include "bullets.h"
#include "game.h"

/**[txh]********************************************************************

  Description: Main loop of the game. Takes care of syncronizing all
  according to the tick counter.

***************************************************************************/

void     run_game (void)
{
   int old_counter, dif, f;

   score = flag = movx = life = draw_table = 0;
   ammo[0] = ammo[1] = MAX_AMMO;
   level = 1;
   MAKE_SHIP (WHITE);
   posx = SCALE_I2F (SIM_SIZE / 2);
   delay[0] = delay[1] = reload[0] = reload[1] = TIME_RELOAD;

   for (f = 0; f < SUB_LINES; f++)
      EN_F.used = 0;

   for (f = 0; f < MAX_MINES; f++)
      MI_F.used = 0;

   tick = old_counter = 0;

   while (!GET (flag, QUIT_GAME | GAME_OVER))
   {
      do
      {
         read_input();
      } while (tick == old_counter);
      
      dif = tick - old_counter;
      old_counter = tick;

      while (dif)
      {
         logic_loop();
         dif--;
      }

      draw_game();
   }
}

/**[txh]********************************************************************

  Description: Transforms any useful user input into abstract variables
  ready for use by the logic routines.

***************************************************************************/


void     read_input (void)
{
   if (key[KEY_ESC])
      SET (flag, QUIT_GAME);

   movx = 0;
  
   if (key[KEY_RSHIFT])
      movx++;

   if (key[KEY_LSHIFT])
      movx--;

   fire[0] = fire[1] = 0;

   if (key[KEY_LCONTROL])
      fire[0]++;
      
   if (key[KEY_RCONTROL])
      fire[1]++;
}

/**[txh]********************************************************************

  Description: Loogic loop of the game. It should be called for each
  elapsed "tick". Moves the ship, the subs, the mines, detects collisions,
  throws more subs into the game, etc.

***************************************************************************/

void     logic_loop (void)
{
   int val;

   if (life >= LIFE)
   {
      /* TODO game over message */
      SET (flag, GAME_OVER);
      SHOW_OUT ("game_over");
      return ;
   }
   
   /* Let's move the ship */
   posx += deltax * movx;

   posx = MAX (posx, lowx);
   posx = MIN (posx, highx);

   /* Fire loads? */
   for (val = 0; val < 2; val ++)
   {
      if (fire[val] && ammo[val] && delay[val] > FIRE_DELAY)
      {
         drop_mine (val);
         if (ammo[val] == MAX_AMMO) reload[val] = 0;
         ammo[val]--;
         delay[val] = draw_table = 0;
      }

      /* Recharge ? */
      if (reload[val] > TIME_RELOAD && ammo[val] < MAX_AMMO)
      {
         ammo[val]++;
         draw_table = reload[val] = 0;
      }
   }

   /* Let's move mines */
   move_mines();

   /* Let's AI some subs :-) */
   move_subs ();

   /* Do we have to add more subs? */
   if (!free_sub_lines())
      return ;

   val = rand() % 200*TICKS;

   if (val < 1)
      add_sub();

}


/**[txh]********************************************************************

  Description: Draws a frame of the game, first on the back buffer, then
  blits that to the screen. Very inefficient, but hey, life's like that :-)

***************************************************************************/

void     draw_game (void)
{
   int x, y, f;
   clear_to_color (back, BLACK);
   rect (back, 0, 0, back->w-1, back->h-1, WHITE);

   draw_sprite (back, ship, F2I (posx) - ship->w / 2, ship->h);

   x = ship->h*2 + 1;
   y = x + SCALE_I2I (1);

   rectfill (back, 0, x, back->w-1, y, WHITE);

   y = x + sub->h;

   for (f = 0; f < MAX_MINES; f++)
   {
      if (MI_F.used) /* Let's draw a mine */
      {
         BITMAP *m = mine;

         /* Here we see if it's a load or a mine */
         if (MI_F.used != 1) m = load;
         
         draw_sprite (back, m, F2I (MI_F.x), F2I (MI_F.y));
      }
   }

   for (f = 0; f < SUB_LINES; f++, y += sub->h + 1)
      if (EN_F.used)
         draw_sprite (back, EN_F.bmp, F2I (EN_F.pos) - sub->w / 2, y);

   if (!draw_table)
   {
      generate_table();
      draw_table = 1;
   }
   stretch_sprite (back, table, 0, SCALE_I2I (2), back->w, ship->h - SCALE_I2I (4));

   /* Final hardware part of the routine */
   vsync();
   blit (back, screen, 0, 0, SCREEN_W / 2 - back->w / 2, 0, back->w, back->h);
}


/**[txh]********************************************************************

  Description: This creates the bitmap which will be used to show the points
  and remaining ammunition.

***************************************************************************/

#define LOOP(cannon)             \
{                                 \
   for (f = 0; f < MAX_AMMO; f++)  \
   {                                \
      if (f < ammo[(cannon)])           \
         ustrcat (text, "X ");        \
      else                             \
         ustrcat (text, "  ");          \
   }                                     \
}

void     generate_table (void)
{
   int f;
   char text[30] = " ";

   LOOP (0);
   usprintf (text + 11, "%05d ", score);
   LOOP (1);

   clear (table);
   textout (table, font, text, 0, 0, WHITE);
}

